//Qualquer duvida entrar em contato com:
// Marcelo Braun -> mbraun@ez-poa.com.br
// Silton Sommer -> silton@pucrs.br
// Andre Antoniazzi -> andre@netsul.com.br

unit UAux;

interface
uses UDXF, OpenGL, Math;

type
  TMatrix44 = array [1..4,1..4] of GLFloat;
  TVet4     = array [1..4] of GLFloat;
  TMatrix34 = array [1..3,1..4] of real;

function PtinPolygon (Pt,P1,P2,P3,P4:TPonto3D): boolean;
procedure CalcVetor(A,B:TPonto3D;var V: TPonto3D);
function Modulo (V: TPonto3D): real;
function Produto_Escalar(v1,v2:TPonto3D):Real;
procedure MatrixMultiply(M1: TMatrix44; var P: TPonto3D);
function AtribuiP3D (X,Y,Z:real): TPonto3D;
procedure AtribuiP3D1 (X,Y,Z:real; var P: TPonto3D);
procedure Intersecao (Lp1, Lp2, Pp1, Pp2, Pp3: TPonto3D; var Inter: TPonto3D);
procedure CalCNormal (a,b,c: TPonto3D; var n: TPonto3D);

const
  M_PI = 3.14159265358979323846;

implementation

procedure AtribuiP3D1 (X,Y,Z:real; var P: TPonto3D);
begin
  P.X:= X;
  P.Y:= Y;
  P.Z:= Z;
end;

function AtribuiP3D (X,Y,Z:real): TPonto3D;
var
  P3D: TPonto3D;
begin
  P3D:= TPonto3D.Create;
	P3D.X:= X;
  P3D.Y:= Y;
  P3D.Z:= Z;
  result:= P3D;
end;

//***********************************************************//
//	Data: __/__/____   Autor: ____________________           //
//	Descrio: Multiplica uma matriz 4x4 por um vetor        //
//***********************************************************//

procedure MatrixMultiply(M1: TMatrix44; var P: TPonto3D);
var I,J : Integer;
    VR, Vaux  : TVet4;
    Pt  : TPonto3D;
begin
  VAux[1]:= P.x;
  VAux[2]:= P.y;
  VAux[3]:= P.z;
  VAux[4]:= 1;

  for I:=1 to 4 do
      VR[I]:=M1[1,I]*P.x+
             M1[2,I]*P.y+
             M1[3,I]*P.z+
             M1[4,I];

 AtribuiP3D1(VR[1],VR[2],VR[3],P);
end;

//***********************************************************//
//	Data: __/__/____   Autor: ____________________           //
//	Descrio:                                               //
//***********************************************************//

function Produto_Escalar(v1,v2:TPonto3D):Real;
begin
  result:=(v1.x*v2.x+v1.y*v2.y+v1.z*v2.z);
end;

//***********************************************************//
//	Data: __/__/____   Autor: ____________________           //
//	Descrio:                                               //
//***********************************************************//

function Modulo (V: TPonto3D): real;
var
	aux: real;
begin
	aux:= Power((V.x * V.x + V.y * V.y + V.z * V.z),0.5);
  result:= aux;
end;

//***********************************************************//
//	Data: __/__/____   Autor: ____________________           //
//	Descrio:  Retorna o vetor AB                           //
//***********************************************************//

procedure CalcVetor(A,B:TPonto3D;var V: TPonto3D);
begin
	V.x:= A.x-B.x;
	V.y:= A.y-B.y;
	V.z:= A.z-B.z;
end;

//***********************************************************//
//	Data: __/__/____   Autor: ____________________           //
//	Descrio: Verifica se o ponto PT pertence ao poligono delimitado
//             pelos pontos P1,P2,P3 e P4                    //
//***********************************************************//

function PtinPolygon (Pt,P1,P2,P3,P4:TPonto3D): boolean;
var
	Vt1, Vt2, Vt3, Vt4: TPonto3D;
  A,B,C,D,T: real;
begin
  Vt1:= TPonto3D.Create;
  Vt2:= TPonto3D.Create;
  Vt3:= TPonto3D.Create;
  Vt4:= TPonto3D.Create;

	CalcVetor(Pt,P1,Vt1);
	CalcVetor(Pt,P2,Vt2);
	CalcVetor(Pt,P3,Vt3);
	CalcVetor(Pt,P4,Vt4);

  A:= (Vt1.x*Vt2.x + Vt1.y*Vt2.y + Vt1.z*Vt2.z)/((Modulo(Vt1)*Modulo(Vt2)));
  B:= (Vt2.x*Vt3.x + Vt2.y*Vt3.y + Vt2.z*Vt3.z)/((Modulo(Vt2)*Modulo(Vt3)));
  C:= (Vt3.x*Vt4.x + Vt3.y*Vt4.y + Vt3.z*Vt4.z)/((Modulo(Vt3)*Modulo(Vt4)));
  D:= (Vt4.x*Vt1.x + Vt4.y*Vt1.y + Vt4.z*Vt1.z)/((Modulo(Vt4)*Modulo(Vt1)));

  A:= arccos(A)*180/M_PI;
  B:= arccos(B)*180/M_PI;
  C:= arccos(C)*180/M_PI;
  D:= arccos(D)*180/M_PI;

  T:= A+B+C+D;
  if T>=360 then
  	result:= True
  else
  	result:= False;

	Vt1.Free;
	Vt2.Free;
	Vt3.Free;
	Vt4.Free;
end;

//***********************************************************//
//	Data: __/__/____   Autor: ____________________           //
//	Descrio: Calcula a normal entre os vetores AB e AC     //
//***********************************************************//

procedure CalCNormal (a,b,c: TPonto3D; var n: TPonto3D);
var
  v1,v2: TPonto3D;

begin

	v1:= TPonto3D.Create;
	v2:= TPonto3D.Create;

	v1.x:= a.x - b.x;
	v1.y:= a.y - b.y;
	v1.z:= a.z - b.z;

	v2.x:= a.x - c.x;
	v2.y:= a.y - c.y;
	v2.z:= a.z - c.z;

	n.x:= (v1.z * v2.y) - (v1.y * v2.z);
	n.y:= (v1.x * v2.z) - (v1.z * v2.x);
	n.z:= (v1.y * v2.x) - (v1.x * v2.y);

  V1.Free;
  V2.Free;
end;

//***********************************************************//
//	Data: __/__/____   Autor: ____________________           //
//	Parametros: LP1,LP2: ponto da linha
//              Pp1,Pp2,Pp3: pontos do plano
//              Inter: retorna por referencio o ponto de intersecao, se houver
//***********************************************************//

procedure Intersecao (Lp1, Lp2, Pp1, Pp2, Pp3: TPonto3D; var Inter: TPonto3D);
var
	Matriz: TMatrix34;
  Normal: TPonto3D;
  a,b,c,t: real;

begin
  Normal:= TPonto3D.Create;
  CalcNormal(Pp1,Pp2,Pp3,Normal);

  a:= Normal.X;
  b:= Normal.Y;
  c:= Normal.Z;

  t:= (- a*Lp1.x + a*Pp1.x - b*Lp1.y + b*Pp1.y - c*Lp1.z + c*Pp1.z)/
      (  a*Lp2.x - a*Lp1.x + b*Lp2.y - b*Lp1.y + c*Lp2.z - c*Lp1.z);


  Inter.x:= Lp1.x + t*(Lp2.x-Lp1.x);
  Inter.y:= Lp1.y + t*(Lp2.y-Lp1.y);
  Inter.z:= Lp1.z + t*(Lp2.z-Lp1.z);

  Normal.Free;
end;


end.
